#!/usr/bin/ruby

require 'standard/swrba_logger'           #Logger Class
require 'standard/swrba_def'              #Common Definition Class
require 'net/https'
require 'rexml/document'
require 'rexml/parsers/pullparser'
require 'standard/swrba_util'

#
#=Protocol Library Class
#Version::   1.0
#Copyright:: Copyright (C) FUJITSU LIMITED 2010.
#
#=== Function Contents
#* Representational State Transfer(REST)

class Swrba_Protocol_Ex
  #Return Value
  attr_accessor :return_code
  attr_accessor :http_code
  attr_accessor :xml_data
  
  #Logger
  @logger = nil
  
  #
  #initialize
  #
  def initialize
    # Log Instance Create
    @logger = Swrba_Logger.new(self.class.name, Swrba_Def::SWRBA_FILE_LOG)
    @return_code = Swrba_Def::NORMAL_CODE
    @http_code = 0
    xml_data=nil
  end
  
  #
  #Representational State Transfer(REST) 
  #
  #_hostname_ :: Host Name Or IP Address
  #_port_ :: Port Number
  #_methodname_ :: Method Name
  #_resourcepath_ :: Resource Path
  #_param_ :: Parameter Value
  #_proxy_ :: Proxy Server Name
  #_proxyport_ :: Proxy Port Number
  #_protocoltype_ :: Protocol Type ["http"/"https"]
  #_requestxml_ :: Request XML
  #_contenttype_ :: Content Type
  #_accept_ :: Accept
  #
  #Return Value::Execution result
  #
  def rest_request(hostname, port, methodname, resourcepath, param, proxy, 
                  proxyport, protocoltype, requestxml,
                  contenttype = nil, accept = nil)
    @logger.warn("rest_request:[begin]")
    @logger.warn("rest_request:{hostname=>#{hostname}}")
    @logger.warn("rest_request:{port=>#{port}}")
    @logger.warn("rest_request:{methodname=>#{methodname}}")
    @logger.warn("rest_request:{resourcepath=>#{resourcepath}}")
    @logger.warn("rest_request:{param=>#{param}}")
    @logger.warn("rest_request:{proxy=>#{proxy}}")
    @logger.warn("rest_request:{proxyport=>#{proxyport}}")
    @logger.warn("rest_request:{protocoltype=>#{protocoltype}}")
    @logger.warn("rest_request:{contenttype=>#{contenttype}}")
    @logger.warn("rest_request:{accept=>#{accept}}")

    #parameter check.
    if hostname == nil || hostname == ""
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      @logger.warn("rest_request:hostname None")
      raise "The hostname is a None."
    end
    if port == nil || port == ""
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      @logger.warn("rest_request:port None")
      raise "The port is a None."
    end
    if methodname != "GET" && methodname != "POST" && methodname != "PUT" && methodname != "DELETE"
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      @logger.warn("rest_request:Illegal methodname")
      raise "Illegal methodname."
    end
    if resourcepath == nil || resourcepath == ""
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      @logger.warn("rest_request:resourcepath None")
      raise "The resourcepath is a None."
    end
    if protocoltype != "http" && protocoltype != "https"
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      @logger.warn("rest_request:protocoltype None")
      raise "Illegal protocoltype."
    end
    if requestxml == nil
      param_value = param
    else
      param_value = requestxml
    end
    
    result = rest_request_main(hostname , port, methodname, resourcepath,
     param_value, nil, nil, proxy, proxyport, protocoltype,contenttype, accept)

    @return_code = Swrba_Def::NORMAL_CODE
    @logger.warn("rest_request:[end]")
    return result
  end
  
  #
  #Representational State Transfer(REST)
  #
  #_hostname_ :: Host Name Or IP Address
  #_port_ :: Port Number
  #_methodname_ :: Method Name
  #_resourcepath_ :: Resource Path
  #_param_ :: Parameter Value
  #_basicusername_ :: BASIC Authentication User Name
  #_basicpassword_ :: BASIC Authentication User Password
  #_proxy_ :: Proxy Server Name
  #_proxyport_ :: Proxy Port Number
  #_protocoltype_ :: Protocol Type ["http"/"https"]
  #_requestxml_ :: Request XML
  #_contenttype_ :: Content Type
  #_accept_ :: Accept
  #
  #Return Value::Execution result
  #

  def rest_request_basic(hostname , port, methodname, resourcepath, param,
   basicusername, basicpassword, proxy, proxyport, protocoltype, requestxml,
                                   contenttype = nil, accept = nil)
    @logger.warn("rest_request_basic:[begin]")
    @logger.warn("rest_request_basic:hostname=>#{hostname}}")
    @logger.warn("rest_request_basic:port=>#{port}}")
    @logger.warn("rest_request_basic:methodname=>#{methodname}}")
    @logger.warn("rest_request_basic:resourcepath=>#{resourcepath}}")
    @logger.warn("rest_request_basic:param=>#{param}}")
    @logger.warn("rest_request_basic:basicusername=>#{basicusername}}")
    @logger.warn("rest_request_basic:proxy=>#{proxy}}")
    @logger.warn("rest_request_basic:proxyport=>#{proxyport}}")
    @logger.warn("rest_request_basic:protocoltype=>#{protocoltype}}")
    @logger.warn("rest_request_basic:{contenttype=>#{contenttype}}")
    @logger.warn("rest_request_basic:{accept=>#{accept}}")
    
    #parameter check.
    if hostname == nil || hostname == ""
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      @logger.warn("rest_request_basic:hostname None")
      raise "The hostname is a None."
    end
    if port == nil || port == ""
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      @logger.warn("rest_request_basic:port None")
      raise "The port is a None."
    end
    if methodname != "GET" && methodname != "POST" && methodname != "PUT" && methodname != "DELETE"
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      @logger.warn("rest_request_basic:Illegal methodname")
      raise "Illegal methodname."
    end
    if resourcepath == nil || resourcepath == ""
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      @logger.warn("rest_request_basic:resourcepath None")
      raise "The resourcepath is a None."
    end
    if protocoltype != "http" && protocoltype != "https"
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      @logger.warn("rest_request_basic:Illegal protocoltype")
      raise "Illegal protocoltype."
    end
    if requestxml == nil
      param_value = param
    else
      param_value = requestxml
    end
    
    result = rest_request_main(hostname , port, methodname, resourcepath, 
       param_value, basicusername, basicpassword, proxy, proxyport, 
                                 protocoltype,contenttype,accept)


    @return_code = Swrba_Def::NORMAL_CODE
    @logger.warn("rest_request_basic:[end]")
    return result
  end
  
  
  #-----------------------------------------------------------------------------
  private
  #-----------------------------------------------------------------------------
  
  #
  #Representational State Transfer(REST) 
  #
  #_hostname_ :: Host Name Or IP Address
  #_port_ :: Host Port Number
  #_methodname_ :: Method Name
  #_resourcepath_ :: Resource Path
  #_param_ :: Parameter Value
  #_basicusername_ :: BASIC Authentication User Name
  #_basicpassword_ :: BASIC Authentication User Password
  #_proxy_ :: Proxy Server Name
  #_proxyport_ :: Proxy Port Number
  #_protocoltype_ :: Protocol Type ["http"/"https"]
  #_requestxml_ :: Request XML
  #_contenttype_ :: Content Type
  #_accept_ :: Accept
  #
  #Return Value::Execution result
  #
  def rest_request_main(hostname , port, methodname, resourcepath, param,
    basicusername, basicpassword, proxy, proxyport, 
    protocoltype, contenttype, accept)

    @xml_data = ""
    @http_code = 0
    begin
      #Recommended is Version 1.2
      Net::HTTP.version_1_2
      
      case methodname
        when "GET"
          @logger.info("rest_request_main:The method name is GET.")
          if param == nil || param == ""
            request = [Net::HTTP::Get.new(resourcepath, 'Accept' => "#{accept}")]
          else
            request = [Net::HTTP::Get.new("#{resourcepath}?#{param}", 
                                                        'Accept' => "#{accept}")]
          end
        when "POST"
          @logger.info("rest_request_main:The method name is POST.")
          request = [Net::HTTP::Post.new(resourcepath), param]
        when "PUT"
          @logger.info("rest_request_main:The method name is PUT.")
          request = [Net::HTTP::Put.new(resourcepath), param]
        when "DELETE"
          @logger.info("rest_request_main:The method name is DELETE.")
          request = [Net::HTTP::Delete.new(resourcepath)]
      end

      request[0].set_content_type(contenttype)
      if basicusername != nil && basicusername != "" && basicpassword != nil
        @logger.info("rest_request_main:BASIC Authentication User:#{basicusername}.")
        request[0].basic_auth(basicusername, basicpassword)
      end
      
      if proxy != nil && proxy != "" && proxyport != nil && proxyport != ""
        @logger.info("rest_request_main:By way of proxy.")
        http_protocol = Net::HTTP::Proxy(proxy, proxyport).new(hostname, port)
      else
        http_protocol = Net::HTTP.new(hostname, port)
      end
      
      if protocoltype == "http"
        @logger.info("rest_request_main:The protocol is HTTP.")
      elsif protocoltype == "https"
        @logger.info("rest_request_main:The protocol is HTTPS.")
        http_protocol.use_ssl = true
        http_protocol.verify_mode = OpenSSL::SSL::VERIFY_NONE
      end
      
      http_protocol.start { |http|
        @logger.info("rest_request_main:Request Start.")
        response = http.request(*request)
        case methodname
          when "GET"
            @xml_data = response.body.to_s
          when "POST"
            @xml_data = response.body.to_s
          when "PUT"
            @xml_data = response.body.to_s
          when "DELETE"
            @xml_data = response.body.to_s
        end
        
        @http_code = response.code.to_i
        
        @logger.info("rest_request_main:Request End.")
      }
    rescue SocketError
      @return_code = Swrba_Def::ERROR_CONNECTION_ABORT
      @logger.warn("rest_request_main:[error]")
      raise
    rescue Errno::ECONNABORTED
      @return_code = Swrba_Def::ERROR_CONNECTION_ABORT
      @logger.warn("rest_request_main:[error]")
      raise
    rescue Errno::ECONNREFUSED
      @return_code = Swrba_Def::ERROR_CONNECTION_ABORT
      @logger.warn("rest_request_main:[error]")
      raise
    rescue Errno::ECONNRESET
      @return_code = Swrba_Def::ERROR_DISCONNECTION
      @logger.warn("rest_request_main:[error]")
      raise
    rescue
      @return_code = Swrba_Def::ERROR_RETRY
      @logger.warn("rest_request_main:[error]")
      raise
    end
    
    if @http_code >= 300 || 200 > @http_code
      @return_code = Swrba_Def::ERROR_RETRY
      @logger.warn("rest_request_main:xml_data=>[#{@xml_data}]")
      @logger.warn("rest_request_main:[error]")
      raise "HTTP Status #{http_code.to_s}. detail=#{@xml_data}"
    end
    
    return @xml_data
  end
end
