#! /usr/bin/ruby
require 'net/https'
require 'csv'
require 'rexml/document'
require 'rexml/parsers/pullparser'
require 'standard/swrba_logger'  #Logger Class
require 'standard/swrba_def'     #Common Definition Class
require 'standard/swrba_cmdb'
require 'swrba_protocol_ex'
require 'swrba_vserver_msgdef'

#=================================================
#=Vitual Server Library Class
#Version::   1.0
#Copyright:: Copyright (C) FUJITSU LIMITED 2010.
#
#=== Function Contents
#* Operating virtual server class
#
#=================================================

# 
# Protocol Matrix
# 
#   [ METHODNAME, RESOURCEPATH,  PARMS,
#   PROXY, PROXYPORT, PROTOCOLTYPE, nil,
#   CONTENTTYPE, ACCEPT]
#

SWRBA_PROTO_LSER = ["GET", "/l_servers", "verbose=false",
                    nil, 0 , "https", nil,
                    "application/xml", "application/xml"]
                 
SWRBA_PROTO_TASK = ["GET", "/tasks/", nil, 
                    nil, 0 , "https", nil,
                    "application/xml", "application/xml"]

SWRBA_PROTO_SNAP = ["GET", "/server_images","mode=snapshot",
                    nil, 0 , "https", nil,
                    "application/xml", "application/xml"]
                    
SWRBA_PROTO_IMG = ["GET", "/server_images", "mode=cloning",
                    nil, 0 , "https", nil,
                    "application/xml", "application/xml"]
SWRBA_PROTO_TEMP = ["GET", "/templates/l_servers", nil, 
                    nil, 0 , "https", nil,
                    "application/xml", "application/xml"]
SWRBA_PROTO_NETID = ["GET", "/networks", nil, 
                    nil, 0 , "https", nil,
                    "application/xml", "application/xml"]

SWRBA_PROTO_LSERST = ["GET", "/l_servers", "verbose=true",
                     nil, 0 , "https", nil,
                    "application/xml", "application/xml"]
# 
# XPATH
# 
SWRBA_XPATH_LSER_HEAD = "/LServers/LServer[@name='"
SWRBA_XPATH_LSER_TAIL = "']"
SWRBA_XPATH_PSTATUS = "/Status/PowerStatus"
SWRBA_XPATH_DISKS = "/Disks/Disk/DiskLink"
SWRBA_XPATH_SNAP = "/ServerImages/ServerImage"
SWRBA_XPATH_LSERVER_HEAD = "/LServers/LServer[@name='"
SWRBA_XPATH_LSERVER_TAIL = "']"
SWRBA_XPATH_TASKID = "/task/id"
SWRBA_XPATH_PROGRESS = "/task/progress"
SWRBA_XPATH_STATUS   = "/task/status"
SWRBA_XPATH_IMG_HEAD = "/ServerImages/ServerImage[@name='"
SWRBA_XPATH_IMG_TAIL = "']"
SWRBA_XPATH_TEMP_HEAD = "/LServerTemplates/LServerTemplate[@name='"
SWRBA_XPATH_TEMP_TAIL = "']"
SWRBA_XPATH_NET="Networks/Network"
SWRBA_XPATH_IDATTR = "id"
SWRBA_XPATH_NAMEATTR = "name"
SWRBA_XPATH_ERROR_CODE = "/error/code"
SWRBA_XPATH_ERROR_MESSAGE = "/error/message"
SWRBA_XPATH_ERROR_ARG = "/error/arg"
SWRBA_XPATH_ERROR_TASK_CODE = "/task/error/code"
SWRBA_XPATH_ERROR_TASK_MESSAGE = "/task/error/message"
SWRBA_XPATH_ERROR_TASK_ARG = "/task/error/arg"
 

# 
# For CMDB
# 

SWRBS_CMDB_HOSTNAME = "127.0.0.1"

SWRBA_SPATH_MHOST = "/%InstalledSoftware[./#observed/rc:InstalledSoftware/" +
  "@productName = 'ServerView Resource Orchestrator']/&GRelation/%LogicalServer"
SWRBA_XPATH_MHOSTIP="entities/cmdb:item/cmdb:record/ns0:" +
                  "LogicalServer/attribute::ipAddress"
SWRBA_XPATH_MHOST="entities/cmdb:item/cmdb:record/ns0:" +
                  "LogicalServer/attribute::hostname"

SWRBA_SPATH_MPORT="/%InstalledSoftware[./#observed/rc:InstalledSoftware/" +
                  "@productName = 'ServerView Resource Orchestrator']"
SWRBA_XPATH_MPORT="entities/cmdb:item/cmdb:record/ns0:" +
                  "InstalledSoftware/attribute::specialNote"


SWRBA_SPATH_USER_HEAD= "/%UserAccount[./#expected][(./#expected/" +
                  "{urn:xmlns-fujitsu-com:rcxml2009-01-swcmdb}UserAccount"
                
SWRBA_SPATH_USER_TAIL="[matches(@ipAddress,'%s')])and(./#expected/" +
                  "{urn:xmlns-fujitsu-com:rcxml2009-01-swcmdb}UserAccount" +
                  "[matches(@isAdminUser,'true')])]"

SWRBA_XPATH_US_ID= "//{urn:xmlns-fujitsu-com:rcxml2009-01-swcmdb}UserAccount/" + 
                   "attribute::userName"
SWRBA_XPATH_US_PASS= "//{urn:xmlns-fujitsu-com:rcxml2009-01-swcmdb}UserAccount" +
                     "/attribute::password"



# 
# Process Kind
#


class Swrba_Vserver

  #--------------------
  # Instance variable
  #--------------------
  attr_accessor :return_code      # Return Value
  @logger = nil                   # Logger
  @vservername = nil              # Virtual server name
  @manageservername = nil         # Manage server name
  @manageserverport = -1          # Manage server port
  @protocolex = nil               # REST
  @taskid = nil                   # task ID
  attr_accessor :err_msg          # error massage
  @process_kind = nil
  @basicusername = nil
  @basicpassword = nil
  

  #--------------------
  # Processing Kind
  #--------------------
  SWRBA_KIND_CREATE   = 0
  SWRBA_KIND_DELETE   = 1
  SWRBA_KIND_START    = 2
  SWRBA_KIND_STOP     = 3
  SWRBA_KIND_RESTART  = 4
  SWRBA_KIND_CHANGE   = 5
  SWRBA_KIND_LIST     = 6
  SWRBA_KIND_DETAIL   = 7
  SWRBA_KIND_BACKUP   = 8
  SWRBA_KIND_RESTORE  = 9


  # 
  # Return Code
  #
  SWRBA_RETURNCODE190 = 171
  SWRBA_RETURNCODE191 = 172
  SWRBA_RETURNCODE192 = 173
  
  #--------------------
  #initialize
  #--------------------
  def initialize (kind)
    # Log Instance Create
    @logger = Swrba_Logger.new(self.class.name, Swrba_Def::SWRBA_FILE_LOG)
    
    # Protocol Instance Create
    @protocolex = Swrba_Protocol_Ex.new
    
    # Set attr_accessor
    @return_code = Swrba_Def::NORMAL_CODE
    
    @process_kind = kind
    
    @err_msg = ""
    
    @manageserverport = -1
  end
  
  #---------------------------------------
  # Register virtual server name 
  #
  #_hostname_ :: Host name or IP address
  #
  #Return Value::None
  #----------------------------------------
  def setVserverName(hostname)
    # Start trace 
    @logger.warn("setVserverName:[begin]")
    @logger.warn("setVserverName:{hostname=>#{hostname}}")

    #Parameters check.
    if hostname == nil || hostname == ""
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      create_errmsg(@return_code, nil)
      @logger.warn("setVserverName:hostname None")
      raise "The hostname is a None."
    end

    # set server name
    @vservername = hostname
    
    # End trace & return
    @return_code = Swrba_Def::NORMAL_CODE
    @logger.warn("setVserverName:[end]")
  end
  
  #
  # getManageHostList
  #
  #Return Value::Execution result
  #
  def getManageHostList()
    # Start trace 
    @logger.warn("getManageHostList:[begin]")
    returnvalues = nil

    ############
    # GET manageserverip & port  from CMDB WHERE vservername
    ############
    #Configuration Item View
    cmdb = Swrba_CMDB.new
    
    if Swrba_Util.check_windows == 1
        ostype = Swrba_Def::OS_TYPE_WINDOWS
    else
        ostype = Swrba_Def::OS_TYPE_LINUX
    end

    result = nil

    begin
      result = cmdb.get_info(SWRBA_SPATH_MHOST, SWRBA_XPATH_MHOSTIP, 
               SWRBS_CMDB_HOSTNAME, 
               nil, nil, nil, nil, ostype)
    rescue => exc
      if exc.message == "Illegal searchxpath."
        returnvalues = [[""],[""],[""]]
        return returnvalues
      end
      @return_code = cmdb.return_code
      create_errmsg(@return_code, nil)
      @logger.warn(exc)
      @logger.warn("getManageHostList:[error]")
      raise exc
    end
    
    mhostipadresses = CSV.parse(result)
    @logger.warn(result)
    
    ############
    # GET manageserverport  from CMDB WHERE vservername
    ############
    #Configuration Item View
    cmdb = Swrba_CMDB.new
    result = nil
    begin
      result = cmdb.get_info(SWRBA_SPATH_MPORT, SWRBA_XPATH_MPORT, 
               SWRBS_CMDB_HOSTNAME, 
               nil, nil, nil, nil, ostype)
      mports = CSV.parse(result)
      @logger.warn(result)
    rescue => exc
      if exc.message == "Illegal searchxpath."
        mports = Array.new(1)
        mports[0] = Array.new(mhostipadresses[0].size,"")
      else
        @return_code = cmdb.return_code
        create_errmsg(@return_code, nil)
        @logger.warn(exc)
        @logger.warn("getManageHostList:[error]")
        raise exc
      end
    end

    if mhostipadresses[0].size != mports[0].size
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      create_errmsg(@return_code, nil)
      @logger.warn(exc)
      @logger.warn("getManageHostList:[error]")
      raise exc
    end

    ############
    # GET manageservername   from CMDB WHERE vservername
    ############
    #Configuration Item View
    result = nil
    begin
      result = cmdb.get_info(SWRBA_SPATH_MHOST, SWRBA_XPATH_MHOST, 
               SWRBS_CMDB_HOSTNAME, 
               nil, nil, nil, nil, ostype)
      mhostnames = CSV.parse(result)
      @logger.warn(result)
    rescue => exc
      if exc.message == "Illegal searchxpath."
        mhostnames = Array.new(1)
        mhostnames[0] = Array.new(mhostipadresses[0].size,"")
      else
        @return_code = cmdb.return_code
        create_errmsg(@return_code, nil)
        @logger.warn(exc)
        @logger.warn("getManageHostList:[error]")
        raise exc
      end
    end
    if mhostipadresses[0].size != mhostnames[0].size
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      create_errmsg(@return_code, nil)
      @logger.warn("getManageHostList:[error]")
      raise exc
    end
    returnvalues = [mhostipadresses[0], mports[0], mhostnames[0]]
 
     @logger.warn("getManageHostList:[end]")
    return returnvalues
  end
  
  
  #
  # Get Manage name
  #
  #_hostname_ :: Host Name Or IP Address
  #
  #Return Value::Execution result
  #
  def getManageHost()
    # Start trace 
    @logger.warn("getManageHost:[begin]")
    
    if @manageservername != nil && @manageserverport != -1
      # return
      @logger.warn("getManageHost:[end]")
      @return_code = Swrba_Def::NORMAL_CODE
      return @manageservername
    end

    #parameter check.
    if @vservername == nil || @vservername == ""
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      create_errmsg(@return_code, nil)
      @logger.warn("getManageHost:virtual server name none")
      raise "virtual server name none."
    end
    
    target = nil
    if @manageservername != nil && @manageserverport == -1
       target=@manageservername
    end
    
    # Get host name list  CMDB
    hostlist = getManageHostList()
    mhostnames = hostlist[0]
    mports = hostlist[1]
    
    tmp_errmsg = nil
    tmp_return_code = nil
    
    i = 0
    while i < mhostnames.size

      if target != nil
        if target == mhostnames[i]
          @manageserverport = mports[i]
          break
        else
          i += 1
          next
        end
      else
        @manageservername = mhostnames[i]
        @manageserverport = mports[i]
        @basicusername = nil
        @basicpassword = nil
      end
      
      if @manageservername == nil || @manageserverport == nil ||
         @manageservername == "" || @manageserverport == ""
        i += 1
        next
      end

      begin
        # REST Request start.
        return_xml = requestREST(
                     SWRBA_PROTO_LSER[0], SWRBA_PROTO_LSER[1], 
                     SWRBA_PROTO_LSER[2], SWRBA_PROTO_LSER[3], 
                     SWRBA_PROTO_LSER[4], SWRBA_PROTO_LSER[5],
                     SWRBA_PROTO_LSER[6], SWRBA_PROTO_LSER[7], 
                     SWRBA_PROTO_LSER[8])
      rescue => exc
        tmp_errmsg = @err_msg
        tmp_return_code = @return_code
        i += 1
        next
      end
      
      # Document Instance Create
      begin
        doc = REXML::Document.new(return_xml)
      rescue => exc
        # XML liblary error
        @return_code = Swrba_Def::ERROR_RETRY
        create_errmsg(@return_code, nil)
        @logger.warn(exc)
        @logger.warn("getManageHost:[error]")
        raise exc
      end
      
      # Get Version Info
      lserver = SWRBA_XPATH_LSERVER_HEAD + @vservername + 
                                               SWRBA_XPATH_LSERVER_TAIL
      if doc.elements[lserver] != nil
        break
      else
        @logger.warn("virtual server name none #{@manageservername},#{@vservername}")
        @logger.warn("virtual server name none return_xml=>#{return_xml}")
      end
      i += 1
    end
    if i == mhostnames.size
      if tmp_errmsg == nil
        @return_code = Swrba_Def::ERROR_RETRY
        create_errmsg(@return_code, nil)
      else
        @return_code = tmp_return_code
        @err_msg = tmp_errmsg
      end
      @logger.warn(exc)
      @logger.warn("getManageHost:[error]")
      raise "manage server not found."
    end

    @logger.warn("getManageHost:[end]")
    return @manageservername
  end


  
  #
  # Get Manage server port
  #
  #Return Value::Execution result
  #
  def getManagePort()
    # Start trace 
    @logger.warn("getManagePort:[begin]")

    if @manageserverport == -1
      hostname = getManageHost()
    end

    # End trace & return
    @return_code = Swrba_Def::NORMAL_CODE
    @logger.warn("getManagePort:[end]")
    return @manageserverport
  end
  
  
  #
  # Get resource ID
  #
  #Return Value::Execution result
  #
  def getResourceID()
    # Start trace 
    @logger.warn("getResourceID:[begin]")

    #parameter check.
    if @vservername == nil || @vservername == ""
      @return_code = Swrba_Def::ERROR_INPUT_DATA
      create_errmsg(@return_code, nil)
      @logger.warn("rest_request:vitual server None")
      raise "The vitual server is a None."
    end
    
    begin
      # REST Request start.
      return_xml = requestREST(
                     SWRBA_PROTO_LSER[0], SWRBA_PROTO_LSER[1], 
                     SWRBA_PROTO_LSER[2], SWRBA_PROTO_LSER[3], 
                     SWRBA_PROTO_LSER[4], SWRBA_PROTO_LSER[5],
                     SWRBA_PROTO_LSER[6], SWRBA_PROTO_LSER[7], 
                     SWRBA_PROTO_LSER[8])
    rescue => exc
      #Communication library AbNormal
      @logger.warn(exc)
      @logger.warn("getResourceID:[error]")
      raise exc
    end
    
    # Document Instance Create
    begin
      doc = REXML::Document.new(return_xml)
    rescue => exc
      # XML liblary error
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(exc)
      @logger.warn("getResourceID:[error]")
      raise exc
    end
    
    # Get Resource ID
    lserver = SWRBA_XPATH_LSERVER_HEAD + @vservername + 
                                               SWRBA_XPATH_LSERVER_TAIL
    if doc.elements[lserver] == nil
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getResourceID:[ResourceID is none error]")
      raise "return xml error"
    end
    data = doc.elements[lserver].attributes[SWRBA_XPATH_IDATTR]
    
    if data == nil || data == ""
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getResourceID:[ResourceID is none error]")
      raise "return xml error"
    end
    
    # End trace & return
    @return_code = Swrba_Def::NORMAL_CODE
    @logger.warn("getResourceID:[end]")
    return data
  end
  
  #
  # Get snapshot ID
  #
  #Return Value::Execution result
  #
  def getSnapshotID()
    # Start trace 
    @logger.warn("getSnapshotID:[begin]")

    # get snapshot-list
    begin
      result_xml = requestREST(
                     SWRBA_PROTO_SNAP[0], SWRBA_PROTO_SNAP[1], 
                     SWRBA_PROTO_SNAP[2], SWRBA_PROTO_SNAP[3], 
                     SWRBA_PROTO_SNAP[4], SWRBA_PROTO_SNAP[5],
                     SWRBA_PROTO_SNAP[6], SWRBA_PROTO_SNAP[7], 
                     SWRBA_PROTO_SNAP[8])
    rescue => exc
      #Communication library AbNormal
      @logger.warn(exc)
      @logger.warn("getSnapshotID:[error]")
      raise exc
    end

    # get snapshot-ID
    begin
      doc = REXML::Document.new(result_xml)
    rescue => exc
      # XML liblary error
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(exc)
      @logger.warn("getSnapshotID:[error]")
      raise exc
    end

    snapshotID = nil
    doc.elements.each(SWRBA_XPATH_SNAP) do |si|
      if si.attributes['name'] == @vservername
        snapshotID = si.attributes['id']
      end
    end
    if snapshotID == nil || snapshotID == ""
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getSnapshotID:[snapshotID is none error]")
      raise "return xml error"
    end

    # End trace & return
    @return_code = Swrba_Def::NORMAL_CODE
    @logger.warn("getSnapshotID:[end]")
    return snapshotID
  end

  #
  # Get getImageID
  #
  #Return Value::Execution result
  #
  def getImageID(image_name)
    # Start trace 
    @logger.warn("getImageID:[begin]")
    begin
      return_xml = requestREST(
                     SWRBA_PROTO_IMG[0], SWRBA_PROTO_IMG[1], 
                     SWRBA_PROTO_IMG[2], 
                     SWRBA_PROTO_IMG[3], 
                     SWRBA_PROTO_IMG[4], SWRBA_PROTO_IMG[5],
                     SWRBA_PROTO_IMG[6], SWRBA_PROTO_IMG[7], 
                     SWRBA_PROTO_IMG[8])
    rescue => exc
      # network exception
      @logger.warn(exc)
      @logger.warn("getImageID:[error]")
      raise exc
    end

    begin
      doc = REXML::Document.new(return_xml)
    rescue => exc
      # XML liblary error
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(exc)
      @logger.warn("getImageID:[error]")
      raise exc
    end
    
    imageID = nil
    if doc.elements[SWRBA_XPATH_IMG_HEAD + image_name + 
      SWRBA_XPATH_IMG_TAIL] == nil
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getImageID:[ImageID is none error]")
      raise "return xml error"
    end

    imageID = doc.elements[SWRBA_XPATH_IMG_HEAD + image_name + 
                  SWRBA_XPATH_IMG_TAIL].attributes[SWRBA_XPATH_IDATTR]
    if imageID == nil || imageID == ""
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getImageID:[imageID is none error]")
      raise "return xml error"
    end

    @logger.warn("getImageID:[end]")
    return imageID
  end
  
  #
  # Get getTemplateID
  #
  #Return Value::Execution result
  #
  def getTemplateID(templatename)
    # Start trace 
    @logger.warn("getTemplateID:[begin]")
    begin
      return_xml = requestREST(
                       SWRBA_PROTO_TEMP[0], SWRBA_PROTO_TEMP[1], 
                       SWRBA_PROTO_TEMP[2], 
                       SWRBA_PROTO_TEMP[3], 
                       SWRBA_PROTO_TEMP[4], SWRBA_PROTO_TEMP[5],
                       SWRBA_PROTO_TEMP[6], SWRBA_PROTO_TEMP[7],
                       SWRBA_PROTO_TEMP[8])
    rescue => exc
      # network exception
      @logger.warn(exc)
      @logger.warn("getTemplateID:[error]")
      raise exc
    end

    begin
      doc = REXML::Document.new(return_xml)
    rescue => exc
      # XML liblary error
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(exc)
      @logger.warn("getTemplateID:[error]")
      raise exc
    end
    
    templateID = nil
    if doc.elements[SWRBA_XPATH_TEMP_HEAD + templatename +
                 SWRBA_XPATH_TEMP_TAIL ] == nil
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getTemplateID:[TemplateID is none error]")
      raise "return xml error"
    end
      
    templateID = doc.elements[SWRBA_XPATH_TEMP_HEAD + templatename +
               SWRBA_XPATH_TEMP_TAIL ].attributes[SWRBA_XPATH_IDATTR]
    if templateID == nil || templateID == ""
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getTemplateID:[error]")
      raise "return xml error"
    end
    @logger.warn("getTemplateID:[end]")
    return templateID
  end

  #
  # Get getNetworkID
  #
  #Return Value::Execution result
  #
  def getNetworkID(netname)
    # Start trace 
    @logger.warn("getNetworkID:[begin]")
    begin
      return_xml = requestREST(
                       SWRBA_PROTO_NETID[0], SWRBA_PROTO_NETID[1], 
                       SWRBA_PROTO_NETID[2], 
                       SWRBA_PROTO_NETID[3], 
                       SWRBA_PROTO_NETID[4] , SWRBA_PROTO_NETID[5],
                       SWRBA_PROTO_NETID[6], SWRBA_PROTO_NETID[7], 
                       SWRBA_PROTO_NETID[8])
    rescue => exc
      # network exception
      @logger.warn(exc)
      @logger.warn("getNetworkID:[requestREST error]")
      raise exc
    end

    begin
      doc = REXML::Document.new(return_xml)
    rescue => exc
      # XML liblary error
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(exc)
      @logger.warn("getNetworkID:[error]")
      raise exc
    end
    netID = nil
    doc.elements.each(SWRBA_XPATH_NET) do |element|
      if element.attributes[SWRBA_XPATH_NAMEATTR] == netname
        netID = element.attributes[SWRBA_XPATH_IDATTR]
      end
    end
    
    if netID == nil || netID == ""
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getNetworkID:[NetworkID error]")
      raise "return xml error"
    end
    @logger.warn("getNetworkID:[end]")
    return netID
  end

  #
  # getPowerStatus
  #
  #Return Value::Execution result
  #
  def getPowerStatus()
    # Start trace 
    @logger.warn("getPowerStatus:[begin]")
    begin
      return_xml = requestREST(
                       SWRBA_PROTO_LSERST[0], 
                       SWRBA_PROTO_LSERST[1], 
                       SWRBA_PROTO_LSERST[2], SWRBA_PROTO_LSERST[3], 
                       SWRBA_PROTO_LSERST[4], SWRBA_PROTO_LSERST[5],
                       SWRBA_PROTO_LSERST[6], SWRBA_PROTO_LSERST[7], 
                       SWRBA_PROTO_LSERST[8])
    rescue => exc
      # network exception
      @logger.warn(exc)
      @logger.warn("getPowerStatus:[requestREST error]")
      raise exc
    end

    begin
      doc = REXML::Document.new(return_xml)
    rescue => exc
      # XML liblary error
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(exc)
      @logger.warn("getPowerStatus:[error]")
      raise exc
    end
    
    if doc.elements[SWRBA_XPATH_LSER_HEAD + @vservername + 
      SWRBA_XPATH_LSER_TAIL + SWRBA_XPATH_PSTATUS] == nil
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getPowerStatus:[powerstate error]")
      raise "return xml error"
    end
    
    powerstate = nil
    powerstate = doc.elements[SWRBA_XPATH_LSER_HEAD + @vservername + 
      SWRBA_XPATH_LSER_TAIL + SWRBA_XPATH_PSTATUS].get_text.value
    
    if powerstate == nil || powerstate == ""
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getPowerStatus:[powerstate error]")
      raise "return xml error"
    end
    
    @logger.warn("getPowerStatus:[end]")
    return powerstate
  end
  
  #
  # getDiskID
  #
  #Return Value::Execution result
  #
  def getDiskID()
    # Start trace 
    @logger.warn("getDiskID:[begin]")
    begin
      return_xml = requestREST(
                       SWRBA_PROTO_LSERST[0], 
                       SWRBA_PROTO_LSERST[1], 
                       SWRBA_PROTO_LSERST[2], SWRBA_PROTO_LSERST[3], 
                       SWRBA_PROTO_LSERST[4], SWRBA_PROTO_LSERST[5],
                       SWRBA_PROTO_LSERST[6], SWRBA_PROTO_LSERST[7], 
                       SWRBA_PROTO_LSERST[8])
    rescue => exc
      # network exception
      @logger.warn(exc)
      @logger.warn("getDiskID:[requestREST error]")
      raise exc
    end

    begin
      doc = REXML::Document.new(return_xml)
    rescue => exc
      # XML liblary error
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(exc)
      @logger.warn("getDiskID:[error]")
      raise exc
    end

    if doc.elements[SWRBA_XPATH_LSER_HEAD + @vservername + 
      SWRBA_XPATH_LSER_TAIL + SWRBA_XPATH_DISKS] == nil
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getDiskID:[DiskID is none error]")
      raise "return xml error"
    end
    
    diskID = nil
    diskID = doc.elements[SWRBA_XPATH_LSER_HEAD + @vservername + 
      SWRBA_XPATH_LSER_TAIL + SWRBA_XPATH_DISKS].attributes[SWRBA_XPATH_IDATTR]
    
    if diskID == nil || diskID == ""
      @return_code = Swrba_Def::ERROR_RETRY
      create_errmsg(@return_code, nil)
      @logger.warn(return_xml)
      @logger.warn("getDiskID:[DiskID is none error]")
      raise "return xml error"
    end
    
    @logger.warn("getDiskID:[end]")
    return diskID
  end

  #
  #Representational State Transfer(REST)
  #
  #_methodname_ :: Method Name
  #_resourcepath_ :: Resource Path
  #_param_ :: Parameter Value
  #_proxy_ :: Proxy Server Name
  #_proxyport_ :: Proxy Port Number
  #_protocoltype_ :: Protocol Type ["http"/"https"]
  #_requestxml_ :: Request XML
  #
  #Return Value::Execution result
  #
  def requestREST(methodname, resourcepath, param, 
                                 proxy, proxyport, protocoltype, 
                                 requestxml,contenttype, accept)
    # Start trace 
    @logger.warn("requestREST:[begin]")
    
    # Get Host name and port number
    mservername = getManageHost()
    mserverport = getManagePort()
    if @basicusername == nil
      ############
      # GET userid and pathword  from CMDB
      ############
      #Configuration Item View
      cmdb = Swrba_CMDB.new

      result = nil
      if Swrba_Util.check_windows == 1
        ostype = Swrba_Def::OS_TYPE_WINDOWS
      else
        ostype = Swrba_Def::OS_TYPE_LINUX
      end

      spath_tail = sprintf(SWRBA_SPATH_USER_TAIL, mservername)
      begin
        result = cmdb.get_info(SWRBA_SPATH_USER_HEAD + spath_tail, 
                 SWRBA_XPATH_US_ID, 
                 SWRBS_CMDB_HOSTNAME, 
                 nil, nil, nil, nil, ostype)
      rescue => exc
        if exc.message == "Illegal searchxpath."
          @logger.warn(exc)
          @logger.warn("requestREST:[cmdb no username error. mservername=>#{mservername}]")
          @return_code = Swrba_Def::ERROR_RETRY
          create_errmsg(@return_code, nil)
        else
          @logger.warn(exc)
          @logger.warn("requestREST:[error]")
          @return_code = cmdb.return_code
          create_errmsg(@return_code, nil)
          raise exc
        end
      end
      
      result_args = CSV.parse(result)
      @basicusername = result_args[0][0]
      result = nil
      begin
        result = cmdb.get_info(SWRBA_SPATH_USER_HEAD + spath_tail,
                 SWRBA_XPATH_US_PASS, 
                 SWRBS_CMDB_HOSTNAME, 
                 nil, nil, nil, nil, ostype)
      rescue => exc
        if exc.message == "Illegal searchxpath."
          @logger.warn(exc)
          @logger.warn("requestREST:[cmdb no password error. mservername=>#{mservername}]")
          @return_code = Swrba_Def::ERROR_RETRY
          create_errmsg(@return_code, nil)
        else
          @logger.warn(exc)
          @logger.warn("requestREST:[error]")
          @return_code = cmdb.return_code
          create_errmsg(@return_code, nil)
          raise exc
        end
      end
      
      result_args = CSV.parse(result)
      @basicpassword = result_args[0][0]
    end
    
    # password decrypt
    b_password = decrypt(@basicpassword)

    # request_basic (BASIC)
    begin
      result = @protocolex.rest_request_basic(mservername ,
         mserverport, methodname, resourcepath, param, @basicusername, 
         b_password, proxy, proxyport, protocoltype, requestxml,
         contenttype, accept)
    rescue Errno::ETIMEDOUT:
      @return_code = Swrba_Def::ERROR_CONNECTION_ABORT
      create_errmsg(@return_code, nil)
      raise
    rescue => exc
      @return_code = @protocolex.return_code
      @logger.warn("requestREST:{http_code=>#{@protocolex.http_code}}")
      if @return_code == Swrba_Def::ERROR_RETRY
        if @protocolex.http_code > 400 && @protocolex.http_code < 408
          @return_code = SWRBA_RETURNCODE190
        elsif @protocolex.http_code > 300 || 
          (@protocolex.http_code > 0 && @protocolex.http_code < 200)
          @return_code = SWRBA_RETURNCODE191
        end
      end
      create_errmsg(@return_code, @protocolex.xml_data)
      raise exc
    end
    
    # when http_code ==202(async mode)
    # if http_code is 202, get task_id
    if @protocolex.http_code == 202
      # Document Instance Create
      begin
        doc = REXML::Document.new(result)
      rescue => exc
        # XML liblary error
        @return_code = Swrba_Def::ERROR_RETRY
        create_errmsg(@return_code, nil)
        @logger.warn(exc)
        @logger.warn("requestREST:[error]")
        raise exc
      end
      if doc.elements[SWRBA_XPATH_TASKID] == nil
        @return_code = Swrba_Def::ERROR_RETRY
        create_errmsg(@return_code, nil)
        @logger.warn(result)
        @logger.warn("requestREST:[error]")
        raise "return xml error"
      end
      # Get task id
      @taskid = doc.elements[SWRBA_XPATH_TASKID].get_text.value
    end

    # End trace & return
    @return_code = Swrba_Def::NORMAL_CODE
    @logger.warn("requestREST:[end]")
    create_errmsg(0,nil)
    return result
  end
  
  #
  # rest_waitsync
  #
  #
  #Return Value::Execution result
  #

  def rest_waitsync()
    # Start trace 
    @logger.warn("rest_waitsync:[begin]")

    #parameter check.
    if @taskid == nil
       @return_code = Swrba_Def::NORMAL_CODE
       create_errmsg(@return_code, nil)
       @logger.warn("rest_waitsync:[end]")
       return ""
    end

    while true
      # Get state
      begin
        return_xml = requestREST(
                 SWRBA_PROTO_TASK[0], SWRBA_PROTO_TASK[1] + @taskid, 
                 SWRBA_PROTO_TASK[2], 
                 SWRBA_PROTO_TASK[3], SWRBA_PROTO_TASK[4], SWRBA_PROTO_TASK[5],
                 SWRBA_PROTO_TASK[6], SWRBA_PROTO_TASK[7], SWRBA_PROTO_TASK[8])
      rescue => exc
        @logger.warn(exc)
        @logger.warn("rest_waitsync:[error]")
        raise exc
      end
      
      # Document Instance Create
      begin
        doc = REXML::Document.new(return_xml)
      rescue => exc
        # XML liblary error
        @return_code = Swrba_Def::ERROR_RETRY
        create_errmsg(@return_code, nil)
        @logger.warn(exc)
        @logger.warn("rest_waitsync:[error]")
        raise exc
      end
      # Checking SWRBA_XPATH_STATUS
      if doc.elements[SWRBA_XPATH_STATUS] != nil
        data = doc.elements[SWRBA_XPATH_STATUS].get_text.value
      else
        @return_code = Swrba_Def::ERROR_RETRY
        create_errmsg(@return_code, nil)
        @logger.warn("rest_waitsync:[error!] nil")
        raise "return xml error"
      end
      
      if data == "error" 
        @return_code = SWRBA_RETURNCODE191
        create_errmsg(@return_code, return_xml)
        @logger.warn(return_xml)
        @logger.warn("rest_waitsync:error ")
        @logger.warn("rest_waitsync:[error]")
        
        raise "request error"
      elsif data == "cancelled"
        @return_code = SWRBA_RETURNCODE191
        create_errmsg(@return_code, return_xml)
        @logger.warn("rest_waitsync:cancelled")
        @logger.warn("rest_waitsync:[error]")
        raise "request cancelled"
      end
      
      @logger.warn("rest_waitsync: status = " + data)

      # Checking SWRBA_XPATH_PROGRESS
      if doc.elements[SWRBA_XPATH_PROGRESS] == nil
        @return_code = Swrba_Def::ERROR_RETRY
        create_errmsg(@return_code, nil)
        @logger.warn(result)
        @logger.warn("rest_waitsync:[error]")
        raise "request abort"
      end
      
      data = doc.elements[SWRBA_XPATH_PROGRESS].get_text.value
      
      if data == "100"
        @return_code = Swrba_Def::NORMAL_CODE
        @logger.warn("rest_waitsync:[end]")
        return return_xml
      end
      @logger.warn("rest_waitsync: status = " + data)
      
      # sleep 3 seconds
      sleep(3)
    end
    @logger.warn("rest_waitsync:[end]")
  end
  
  
  #
  # The virtual server name is registered.
  #
  #_hostname_ :: Host Name Or IP Address
  #
  #Return Value::Execution result
  #
  def setManageHost(host_name)
    @logger.warn("setManageHost:[begin]")
    
    @manageservername = host_name
    @basicusername = nil
    @basicpassword = nil
    
    @logger.warn("setManageHost:{host_name=>#{host_name}}")
    @logger.warn("setManageHost:[end]")
  end

  #
  # The virtual server port is registered.
  #
  #_hostname_ :: Host Name Or IP Address
  #
  #Return Value::Execution result
  #
  def setManagePort(port_number)
    @logger.warn("setManagePort:[begin]")
    
    @manageserverport = port_number
    
    @logger.warn("setManagePort:{port_number=>#{port_number}}")
    @logger.warn("setManagePort:[end]")
  end

  #
  # create_errmsg
  #
  #_rcode_ ::  return_code
  #_xml_   ::  xmldata
  #
  # Return Value::Err message
  #
  def create_errmsg(rcode, xml)
    # Start trace 
    @logger.warn("create_errmsg:[begin]")
    @logger.warn("create_errmsg:{rcode=>#{rcode.to_s}}")
    @logger.warn("create_errmsg:{xml=>#{xml}}")
    
    errorcode = nil
    errormsg = nil
    if xml != nil
      begin
        doc = REXML::Document.new(xml)
        
        if doc.elements[SWRBA_XPATH_ERROR_CODE] != nil
          errorcode = doc.elements[SWRBA_XPATH_ERROR_CODE].get_text.value
          errormsg = doc.elements[SWRBA_XPATH_ERROR_MESSAGE].get_text.value
          cnt = 0
          doc.elements.each(SWRBA_XPATH_ERROR_ARG) do |element|
            errormsg = errormsg.sub("${#{cnt}}", element.get_text.value)
            cnt += 1
          end
        elsif doc.elements[SWRBA_XPATH_ERROR_TASK_CODE] != nil
          errorcode = doc.elements[SWRBA_XPATH_ERROR_TASK_CODE].get_text.value
          errormsg = doc.elements[SWRBA_XPATH_ERROR_TASK_MESSAGE].get_text.value
          cnt = 0
          doc.elements.each(SWRBA_XPATH_ERROR_TASK_ARG) do |element|
            errormsg = errormsg.sub("${#{cnt}}", element.get_text.value)
            cnt += 1
          end
        end
      rescue => exc
        # XML liblary error
        errormsg = nil
      end
    end
    
    if rcode == Swrba_Def::NORMAL_CODE
      case @process_kind
        when Swrba_Vserver::SWRBA_KIND_CREATE
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_CREATE
        when Swrba_Vserver::SWRBA_KIND_DELETE
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_DELETE
        when Swrba_Vserver::SWRBA_KIND_START
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_START
        when Swrba_Vserver::SWRBA_KIND_STOP
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_STOP
        when Swrba_Vserver::SWRBA_KIND_RESTART
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_RESTART
        when Swrba_Vserver::SWRBA_KIND_CHANGE
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_CHANGE
        when Swrba_Vserver::SWRBA_KIND_LIST
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_LIST
        when Swrba_Vserver::SWRBA_KIND_DETAIL
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_DETAIL
        when Swrba_Vserver::SWRBA_KIND_BACKUP
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_BACKUP
        when Swrba_Vserver::SWRBA_KIND_RESTORE
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_RESTORE
        else
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_OTHER
      end
      if @process_kind != Swrba_Vserver::SWRBA_KIND_LIST
        if @vservername != nil
          @err_msg = @err_msg + " VirtualServerName=<#{@vservername}>'"
        else
          @err_msg = @err_msg + " VirtualServerName=<>"
        end
      end
    elsif rcode == SWRBA_RETURNCODE190
      @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_RESTERR
      if @process_kind != Swrba_Vserver::SWRBA_KIND_LIST
        if @vservername != nil
          @err_msg = @err_msg + " VirtualServerName=<#{@vservername}>"
        else
          @err_msg = @err_msg + " VirtualServerName=<>"
        end
      end
      if errormsg != nil
        @err_msg = @err_msg + 
        " detail=<ReturnCode='#{errorcode}' Message='#{errormsg}'>"
      else
        @err_msg = @err_msg +
              " detail=<ReturnCode='#{@protocolex.http_code.to_s}' Message=''>"
      end
    elsif rcode == SWRBA_RETURNCODE191
      case @process_kind
        when Swrba_Vserver::SWRBA_KIND_CREATE
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_CREATE
        when Swrba_Vserver::SWRBA_KIND_DELETE
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_DELETE
        when Swrba_Vserver::SWRBA_KIND_START
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_START
        when Swrba_Vserver::SWRBA_KIND_STOP
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_STOP
        when Swrba_Vserver::SWRBA_KIND_RESTART
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_RESTART
        when Swrba_Vserver::SWRBA_KIND_CHANGE
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_CHANGE
        when Swrba_Vserver::SWRBA_KIND_LIST
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_LIST
        when Swrba_Vserver::SWRBA_KIND_DETAIL
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_DETAIL
        when Swrba_Vserver::SWRBA_KIND_BACKUP
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_BACKUP
        when Swrba_Vserver::SWRBA_KIND_RESTORE
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_RESTORE
        else
          @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_OTHER
      end
      if @process_kind != Swrba_Vserver_Msgdef::SWRBA_MSG_INFO_LIST
        if @process_kind != Swrba_Vserver::SWRBA_KIND_LIST
          if @vservername != nil
            @err_msg = @err_msg + " VirtualServerName=<#{@vservername}>"
          else
            @err_msg = @err_msg + " VirtualServerName=<>"
          end
        end
      end
      if errormsg != nil
        @err_msg = @err_msg + 
         " detail=<ReturnCode='#{errorcode}' Message='#{errormsg}'>"
      else
        @err_msg = @err_msg + 
          " detail=<ReturnCode='#{@protocolex.http_code.to_s}' Message=''>"
      end
    elsif rcode == SWRBA_RETURNCODE192
      if @process_kind == Swrba_Vserver::SWRBA_KIND_STOP
        @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_NOTRUNNING
      else
        @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_RUNNING
      end
      if @process_kind != Swrba_Vserver::SWRBA_KIND_LIST
        if @vservername != nil
          @err_msg = @err_msg + " VirtualServerName=<#{@vservername}>"
        else
          @err_msg = @err_msg + " VirtualServerName=<>"
        end
      end
    else
      @err_msg = Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_OCCURED
      if @process_kind != Swrba_Vserver::SWRBA_KIND_LIST
        if @vservername != nil
          @err_msg = @err_msg + " VirtualServerName=<#{@vservername}>"
        else
          @err_msg = @err_msg + " VirtualServerName=<>"
        end
      end
      if (errormsg != nil)
        @err_msg = @err_msg + 
         " detail=<ReturnCode='#{errorcode}' Message='#{errormsg}'>"
      end
    end
    
    @err_msg = @err_msg + "\n"
    
    # End trace & return
    if rcode == Swrba_Def::NORMAL_CODE
      @logger.warn("create_errmsg:[end]")
    else
      @logger.warn("create_errmsg:[end]:#{@err_msg}")
    end
    return @err_msg
  end  
end

#-----------------------------------------------------------------------------
private
#-----------------------------------------------------------------------------
def decrypt(data)
  result = ""
  if data != nil && data != ""
    java = ""
    classpath = ""
    if Swrba_Util.check_windows == 1
      java = "#{ENV['JAVA_HOME']}/bin/java"
      classpath = 
            "#{ENV['ITPM_HOME']}/RBAOPE/bin/parts/lib/standard/swrba_input.jar"
    else
      java = "/opt/FJSVawjbk/jdk5/bin/java"
      classpath = "/opt/FJSVswpm/rbaope/bin/parts/lib/standard/swrba_input.jar"
    end
    java = java.gsub('\\','/')
    classpath = classpath.gsub('\\','/')
    wkstatus, wkstdout, wkstderr = 
                         systemu("#{java} -cp #{classpath} RubyDecrypt #{data}")
    if wkstatus.exitstatus == 0
      result = wkstdout
    else
      result = data
    end
  end
  return result
end


