#!/usr/bin/ruby

require 'kconv'
require 'csv'
require 'rexml/document'
require 'standard/swrba_logger'           # Logger Class
require 'standard/swrba_def'              # Common Definition Class
require 'standard/swrba_input'            # Input Information File Analysis Class
require 'standard/swrba_util'             # Utility Class
require 'swrba_protocol_ex'               # REST Communication Class
require 'swrba_vserver'                   # Vserver Class


#=Combination ROR
#Version::   1.0
#Copyright:: Copyright (C) FUJITSU LIMITED 2010.
#
#=== Function Contents
#* Get Virtual Server list
#* Convert xml-list from ROR format
#* Convert xml-detail from ROR format
#


# Const value
# Get LogicalServer List(REST Communication)
RBAMETHODNAME = "GET"
RBARESOUCEPATH = "/l_servers" 
RBAPARMS = "verbose=false"
RBAPROXY = nil
RBAPROXYPORT = 0
RBAPROTOCOLTYPE = "https"
RBAREQUESTXML = nil
RBACONTENTTYPE = "application/xml"
RBAACCEPT = "application/xml"

class Swrba_Vserverlist

SOFTNAME_ROR = "ServerView Resource Orchestrator"
MSG_CONVERTDETAIL = "Complete to convert Virtual server detail"
MSG_CONVERTLIST = "Complete to convert Virtual server list"

attr_accessor :return_code      # Return Value
attr_accessor :err_msg          # Error Message

@vserver = nil

def initialize(kind)
	# Logger Class Instance
	@logger = Swrba_Logger.new(self.class.name, Swrba_Def::SWRBA_FILE_LOG)
	# Swrba_Vserver class Instance
	@vserver = Swrba_Vserver.new(kind)
	@err_msg = ""
	@return_code = Swrba_Def::NORMAL_CODE
end

# Get Virtual Server list
def getVserverlist
	@logger.warn(":[begin]")
	
	# get ManagerServer list(Array)
	begin
		hostlist = @vserver.getManageHostList()
	rescue => exc
			@logger.warn(exc)
			@err_msg = @vserver.err_msg.toutf8
			@return_code = @vserver.return_code
			raise "getManageHostList error"
	end

	output_xml = REXML::Document.new
	output_xml << REXML::XMLDecl.new('1.0', 'UTF-8')
	
	# header(Resource)
	head = nil
	head = REXML::Element.new("Resources")

	# Get vserver list from all manager
    i = 0
    while i < hostlist[0].size

		mgr_ipaddr = nil
		mgr_ipaddr = hostlist[0][i]
		mgr_port = nil
		mgr_port = hostlist[1][i]
		mgr_name = nil
		mgr_name = hostlist[2][i]
        i += 1

		if mgr_ipaddr == nil || mgr_ipaddr == ""
			@logger.warn("getVserverlist:mgr_ipaddr None")
			next
		end
		if mgr_port == nil || mgr_port == ""
			@logger.warn("getVserverlist:mgr_port None #{mgr_ipaddr}")
			next
		end
		if mgr_name == nil || mgr_name == ""
			@logger.warn("getVserverlist:mgr_name None #{mgr_ipaddr}")
			mgr_name = ""
		end
		
		# Set manager ipaddress & port number
		@vserver.setManageHost(mgr_ipaddr)
		@vserver.setManagePort(mgr_port)
		
		# REST Communication(get vserver list)
		begin
			input_xml = nil
			input_xml = @vserver.requestREST(RBAMETHODNAME, RBARESOUCEPATH, 
			RBAPARMS, RBAPROXY, RBAPROXYPORT, RBAPROTOCOLTYPE, RBAREQUESTXML, 
			RBACONTENTTYPE, RBAACCEPT)
		rescue => exc
			@logger.warn(exc)
			@err_msg = @vserver.err_msg.toutf8
			@return_code = @vserver.return_code
			raise "requestREST error"
		end
		
		# convert to xml from input_xml
		begin
			mgr_server = nil
			mgr_server = convertRORlist(input_xml, mgr_name)
		rescue => exc
			# input_xml exception
			@logger.warn(exc)
			@err_msg = exc.toutf8
			@return_code = Swrba_Def::SWRBA_RETURNCODE191
			STDERR.puts exc.toutf8
			raise "convert error"
		end
		
		head.add_element(mgr_server)
	end
	output_xml.add_element(head)

	output = "" 
    output_xml.write output
	
	# Set error message, return code
	@err_msg = @vserver.create_errmsg(Swrba_Def::NORMAL_CODE, nil)
	@return_code = Swrba_Def::NORMAL_CODE
	@logger.warn(":[end]")
	return output
end


# convert XML (list)
def convertRORlist(input_xml, mgr_name)
	@logger.warn(":[begin]")
	@err_msg = ""
	@return_code = Swrba_Def::NORMAL_CODE
	
	# initilize REXML type
	input_doc = REXML::Document.new(input_xml)
	
	# header(ManagerServer-Element)
	mgr_server = REXML::Element.new("ManagerServer")
	mgr_server.add_attribute("name", mgr_name)
	
	# getName
	name = nil
	name = input_doc.elements["//LServers/Lserver/attribute::name"]
	lsv = nil
	lsv = input_doc.elements["/LServers"]
	REXML::XPath.each( lsv, "LServer/attribute::name"){ |names|
		vsn = nil
		vsn = REXML::Element.new("VirtualServerName")
		vsn.text = names
		mgr_server.add_element(vsn)
	}
	
	@return_code = Swrba_Def::NORMAL_CODE
	@err_msg = MSG_CONVERTLIST
	@logger.warn(":[end]")
	return mgr_server
end

# convert XML(detail)
def convertRORdetail(input_xml, detail_flg)
	@logger.warn(":[begin]")
	@err_msg = ""
	@return_code = Swrba_Def::NORMAL_CODE
	
	# initilize REXML type
	input_doc = REXML::Document.new(input_xml)
	output_doc = REXML::Document.new
	output_doc << REXML::XMLDecl.new('1.0', 'UTF-8')
	
	# header(Resource)
	head = nil
	head = REXML::Element.new("Resources")

	# getName
	vservername = REXML::Element.new("VirtualServerName")
	vservername.text = input_doc.elements["LServer/attribute::name"]
	head.add_element(vservername)

	# getOSType
	ostype = REXML::Element.new("OSType")
	if input_doc.elements["LServer/OSType"] != nil
		ostype.text = input_doc.elements["LServer/OSType"].text
	end
	head.add_element(ostype)

	# getPowerStatus
	powerstatus = REXML::Element.new("PowerStatus")
	if input_doc.elements["LServer/Status/PowerStatus"] != nil
		powerstatus.text = input_doc.elements["LServer/Status/PowerStatus"].text
	end
	head.add_element(powerstatus)

	# getProcessor
	cpu = nil
	cpu = REXML::Element.new("Processor")
	
	tmp_cpu = nil
	tmp_cpu = input_doc.elements["LServer/CPU"]
	
	cpuarch = REXML::Element.new("ProcessorArch")
	if tmp_cpu != nil && tmp_cpu.elements["CPUArch"] != nil
		cpuarch.text = tmp_cpu.elements["CPUArch"].text
	end
	cpu.add_element(cpuarch)
	
	cpuspeed = REXML::Element.new("ProcessorSpeed")
	if tmp_cpu != nil && tmp_cpu.elements["CPUPerf"] != nil
		cpuspeed.text = tmp_cpu.elements["CPUPerf"].text
	end
	cpu.add_element(cpuspeed)
	
	noc = REXML::Element.new("NumOfProcessors")
	if tmp_cpu != nil && tmp_cpu.elements["NumOfCPU"] != nil
		noc.text = tmp_cpu.elements["NumOfCPU"].text
	end
	cpu.add_element(noc)
	
	head.add_element(cpu)

	# getMemory
	memory = nil
	memory = REXML::Element.new("Memory")
	
	memorysize = nil
	memorysize = REXML::Element.new("MemorySize")
	if input_doc.elements["LServer/Memory/MemorySize"] != nil
		memorysize.text = input_doc.elements["LServer/Memory/MemorySize"].text
	end
	memory.add_element(memorysize)
    
	head.add_element(memory)

	# getDisk
	disks = REXML::Element.new("Disks")
	input_doc.elements.each("LServer/Disks/Disk") do |tmp_disk|
		disk = REXML::Element.new("Disk")
		disksize = REXML::Element.new("DiskSize")
		if tmp_disk.elements["DiskSize"] != nil
			disksize.text = tmp_disk.elements["DiskSize"].text
		end
		disk.add_element(disksize)
		disks.add_element(disk)
	end
	head.add_element(disks)

	# getNIC
	nics = REXML::Element.new("Networks")
	input_doc.elements.each("LServer/NICs/NIC") do |tmp_nic|
		nic = REXML::Element.new("Network")
		
		macaddr = REXML::Element.new("MacAddress")
		if tmp_nic.elements["MacAddress"] != nil
			macaddr.text = tmp_nic.elements["MacAddress"].text
		end
		nic.add_element(macaddr)
		
		ipaddr = REXML::Element.new("IpAddress")
		if tmp_nic.elements["IpAddress"] != nil
			ipaddr.text = tmp_nic.elements["IpAddress"].text
		end
		nic.add_element(ipaddr)
		
		nics.add_element(nic)
	end
	head.add_element(nics)
	
	detail_flg = detail_flg.upcase
	if detail_flg == "ON"
		# AdditionalInformation
		tmp = input_doc.elements["LServer"]
		additionalinfo = REXML::Element.new("AdditionalInformation")
		soft = REXML::Element.new("Software")
		soft.text = SOFTNAME_ROR
		additionalinfo.add_element(soft)
		
		outputdata = REXML::Element.new("OutputData")
		elements = ["LServer", "LServer/CPU", "LServer/Disks", "LServer/Disks/Disk",
			"LServer/NICs", "LServer/NICs/NIC", "LServer/HBAs", "LServer/HBAs/HBA",
			"LServer/Current", "LServer/Policy", "LServer/Status", "LServer/From",
			"LServer/Snapshots","LServer/Memory"]
		elements.each {|element|
			input_doc.elements.each(element) do |node|
				if node != nil
					node.texts.each {|text|
						node.delete(text)
					}
				end
			end
		}
		outputdata.add_element(tmp)
		
		additionalinfo.add_element(outputdata)
		head.add_element(additionalinfo)
	end
	
	output_doc.add_element(head)
	
	output = "" 
    output_doc.write output
	
	@return_code = Swrba_Def::NORMAL_CODE
	@err_msg = MSG_CONVERTDETAIL
	@logger.warn(":[end]")
	return output
end

end
