#!/usr/bin/ruby

require 'kconv'
require 'standard/swrba_logger'           # Logger Class
require 'standard/swrba_def'              # Common Definition Class
require 'standard/swrba_input'            # Input Information File Analysis Class
require 'standard/swrba_util'             # Util Library class
require 'swrba_vserver'                   # Vserver Class
require 'swrba_vserver_msgdef'            # vserver message 

#=combination ROR
#Version::   1.0
#Copyright:: Copyright (C) FUJITSU LIMITED 2010.
#
#=== Function Contents
#* change virtual server
#

# Const value
METHODNAME = "PUT"
RESOURCEPATH = "/l_servers/" 
PARMS = nil
PROXY = nil
PROXYPORT = 0
PROTOCOLTYPE = "https"
REQUESTXML = nil
CONTENTTYPE = "application/xml"
CONTENTTYPEURL = "application/x-www-form-urlencoded"
ACCEPT = "application/xml"


SWRBA_CHANGE_XML="<Resources><LServer name='%s'>" +
"<CPU>%s%s%s%s%s%s</CPU>" +
"<Memory>%s%s%s</Memory>" +
"</LServer></Resources>"

SWRBA_CHANGE_CPUPERF_S="<CPUPerf>"
SWRBA_CHANGE_CPUPERF_E="</CPUPerf>"
SWRBA_CHANGE_NUMOFCPU_S="<NumOfCPU>"
SWRBA_CHANGE_NUMOFCPU_E="</NumOfCPU>"
SWRBA_CHANGE_MEMORY_S="<MemorySize>"
SWRBA_CHANGE_MEMORY_E="</MemorySize>"

SWRBA_CREATE_DISKPARAM = "disk_rid=%s&disk_size=%s"


# Logger Class Instance
logger = Swrba_Logger.new("swrba_vserver_changesystem", Swrba_Def::SWRBA_FILE_LOG)

logger.warn(":[begin]")

# Input Information File Analysis Class
input = Swrba_Input.new

# Initialize
option = []

# Get Option Information
while data = ARGV.shift
  option.push(data)
end

# Check Parameter Number
if option.size != 2
  logger.warn("parameter check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check Option Form
if option[0] != "-input"
  logger.warn("option check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check File Exist
if File.exist?(option[1]) == false
  logger.warn("file exist error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Input File Name
logger.warn("Input File Name = " + option[1])

# Get Input Information
begin
  servername = input.input_analysis(option[1], "servername")
  newservername = input.input_analysis(option[1], "newservername")
  processorspeed = input.input_analysis(option[1], "processorspeed")
  numofprocessor = input.input_analysis(option[1], "numofprocessor")
  memorysize = input.input_analysis(option[1], "memorysize")
  disksize = input.input_analysis(option[1], "disksize")
rescue
  logger.warn("input analysis error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_RETRY
end

# Check Required
if servername == nil || servername == ""
  logger.warn("servername None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

changeflg = 0

if newservername == nil || newservername == ""
  newservername = servername
else
  changeflg = 1
end

begin
  if numofprocessor != nil && numofprocessor != ""
    numofprocessor_arg = numofprocessor.to_s.split(/\./)
    if numofprocessor_arg.size != 1
      raise
    end
    if numofprocessor.to_i < 1 || numofprocessor.to_i > 8
      raise
    end
    changeflg = 1
    numofprocessor_s = SWRBA_CHANGE_NUMOFCPU_S
    numofprocessor_e = SWRBA_CHANGE_NUMOFCPU_E
  else
    numofprocessor = ""
    numofprocessor_s = ""
    numofprocessor_e = ""
  end
rescue
  logger.warn("numofprocessor invalid")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end


begin
  if processorspeed != nil && processorspeed != ""
    processorspeed_arg = processorspeed.to_s.split(/\./)
    if processorspeed_arg.size == 1
      if processorspeed_arg[0].to_i < 1 || processorspeed_arg[0].to_i > 8
         raise
      end
    elsif processorspeed_arg.size == 2
      if processorspeed_arg[0].to_i < 0 || processorspeed_arg[0].to_i > 8
         raise
      end
      if processorspeed_arg[1].to_i < 0 || processorspeed_arg[1].to_i > 9
         raise
      end
      if processorspeed_arg[0].to_i == 0 && processorspeed_arg[1].to_i == 0
         raise
      end
      if processorspeed_arg[0].to_i == 8 && processorspeed_arg[1].to_i > 0
         raise
      end
      if processorspeed_arg[1].to_s.length > 1
         raise
      end
    else
      raise
    end
    changeflg = 1
    processorspeed_s = SWRBA_CHANGE_CPUPERF_S
    processorspeed_e = SWRBA_CHANGE_CPUPERF_E
  else
    processorspeed = ""
    processorspeed_s = ""
    processorspeed_e = ""
  end
rescue
  logger.warn("processorspeed invalid")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

begin
  if memorysize != nil && memorysize != ""
    memorysize_arg = memorysize.to_s.split(/\./)
    if memorysize_arg.size == 1
      if memorysize_arg[0].to_i < 1 || memorysize_arg[0].to_i > 255
         raise
      end
    elsif memorysize_arg.size == 2
      if memorysize_arg[0].to_i < 0 || memorysize_arg[0].to_i > 255
         raise
      end
      if memorysize_arg[1].to_i < 0 || memorysize_arg[1].to_i > 9
         raise
      end
      if memorysize_arg[0].to_i == 0 && memorysize_arg[1].to_i == 0
         raise
      end
      if memorysize_arg[0].to_i == 255 && memorysize_arg[1].to_i > 0
         raise
      end
      if memorysize_arg[1].to_s.length > 1
         raise
      end
    else
      raise
    end
    changeflg = 1
    memorysize_s = SWRBA_CHANGE_MEMORY_S
    memorysize_e = SWRBA_CHANGE_MEMORY_E
  else
    memorysize = ""
    memorysize_s = ""
    memorysize_e = ""
  end
rescue
  logger.warn("memorysize invalid")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

begin
  if disksize != nil  && disksize != ""
    disksize_arg = disksize.to_s.split(/\./)
    if disksize_arg.size == 1
      if disksize_arg[0].to_i < 1 || disksize_arg[0].to_i > 2048
         raise
      end
    elsif disksize_arg.size == 2
      if disksize_arg[0].to_i < 0 || disksize_arg[0].to_i > 2048
         raise
      end
      if disksize_arg[1].to_i < 0 || disksize_arg[1].to_i > 9
         raise
      end
      if disksize_arg[0].to_i == 0 && disksize_arg[1].to_i == 0
         raise
      end
      if disksize_arg[0].to_i == 2048 && disksize_arg[1].to_i > 0
         raise
      end
      if disksize_arg[1].to_s.length > 1
         raise
      end
    else
      raise
    end
  end
rescue
  logger.warn("disksize invalid")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end


# create swrba_vserver class

vserver = Swrba_Vserver.new(Swrba_Vserver::SWRBA_KIND_CHANGE)

begin
  # Get Information(Manager-Server) from CMDB
  vserver.setVserverName(servername)

  logger.warn("Managername = " + vserver.getManageHost)
  logger.warn("Managerport = " + vserver.getManagePort.to_s)
rescue => exc
  #  exception
  logger.warn(exc)
  logger.warn(":[setVserverName error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end

## Get Resource-ID
resourceID = nil
begin
  resourceID = vserver.getResourceID()
  logger.warn("resourceID = " + resourceID)
rescue => exc
  #  exception
  logger.warn(exc)
  logger.warn(":[getResourceID error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end

################
## Get Power Status
################
powerstatus = nil
begin
  powerstatus = vserver.getPowerStatus()
rescue => exc
  #  exception
  logger.warn(exc)
  logger.warn(":[getPowerStatus error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end

msg = nil
if powerstatus == "on"
  logger.warn("virtual server status is running")
  msg = vserver.create_errmsg(Swrba_Vserver::SWRBA_RETURNCODE192, nil)
  STDERR.puts msg.toutf8
  exit Swrba_Vserver::SWRBA_RETURNCODE192
end

if changeflg == 1
  ################
  ## Change VServer
  ################
  changexml = nil
  changexml = sprintf(SWRBA_CHANGE_XML, newservername, 
             processorspeed_s, processorspeed.to_s , processorspeed_e,
             numofprocessor_s, numofprocessor.to_s, numofprocessor_e,
             memorysize_s,memorysize.to_s, memorysize_e);
  return_xml = nil
  begin
    return_xml = vserver.requestREST(
                 METHODNAME, RESOURCEPATH + resourceID , PARMS, PROXY, 
                 PROXYPORT, PROTOCOLTYPE, changexml, CONTENTTYPE, ACCEPT)
  rescue => exc
    #  exception
    logger.warn(exc)
    logger.warn(changexml)
    logger.warn(":[requestREST error]")
    STDERR.puts vserver.err_msg.toutf8
    exit vserver.return_code
  end


  ################
  ## Wait Change VServer
  ################
  return_xml = nil
  begin
    return_xml = vserver.rest_waitsync()
  rescue => exc
    #  exception
    logger.warn(exc)
    logger.warn(":[rest_waitsync error]")
    STDERR.puts vserver.err_msg.toutf8
    exit vserver.return_code
  end
end


if disksize != nil  && disksize != ""
  ################
  ## Get Disk ID
  ################
  begin
    vserver.setVserverName(newservername)
  rescue => exc
    #  exception
    logger.warn(exc)
    logger.warn(":[setVserverName error]")
    STDERR.puts vserver.err_msg.toutf8
    exit vserver.return_code
  end
  diskID = nil
  begin
    diskID = vserver.getDiskID()
  rescue => exc
    #  exception
    logger.warn(exc)
    logger.warn(":[getDiskID error]")
    STDERR.puts vserver.err_msg.toutf8
    exit vserver.return_code
  end

  disk_param = nil
  disk_param = sprintf(SWRBA_CREATE_DISKPARAM, diskID.to_s, disksize.to_s)

  ################
  ## Change Disk Size
  ################
  return_xml = nil
  begin
    return_xml = vserver.requestREST(
                 METHODNAME, RESOURCEPATH + resourceID + "/attach",
                  PARMS, PROXY, 
                 PROXYPORT, PROTOCOLTYPE, disk_param, CONTENTTYPEURL, ACCEPT)
  rescue => exc
    #  exception
    logger.warn(exc)
    logger.warn(":[requestREST(disk) error]")
    STDERR.puts vserver.err_msg.toutf8
    exit vserver.return_code
  end


  ################
  ## Wait Disk Size
  ################
  return_xml = nil
  begin
    return_xml = vserver.rest_waitsync()
  rescue => exc
    #  exception
    logger.warn(exc)
    logger.warn(":[rest_waitsync(disk) error]")
    STDERR.puts vserver.err_msg.toutf8
    exit vserver.return_code
  end
end

# successful
STDOUT.print newservername.toutf8
STDERR.puts vserver.err_msg.toutf8
logger.warn(":[end]")

exit Swrba_Def::NORMAL_CODE


