#!/usr/bin/ruby

require 'kconv'
require 'rexml/document'
require 'standard/swrba_logger'           # Logger Class
require 'standard/swrba_def'              # Common Definition Class
require 'standard/swrba_input'            # Input Information File Analysis Class
require 'swrba_vserver'                   # Vserver Class
require 'swrba_vserver_msgdef'            # vserver message 

#=combination ROR
#Version::   1.0
#Copyright:: Copyright (C) FUJITSU LIMITED 2010.
#
#=== Function Contents
#* put snapshot virtual server
#


# Const value

METHODNAME = "PUT"
RESOURCEPATH_HEAD = "/server_images/" 
RESOURCEPATH_TAIL = "/restore"
PARMS = nil
PROXY = nil
PROXYPORT = 0
PROTOCOLTYPE = "https"
REQUESTXML = nil
CONTENTTYPE = "application/xml"
ACCEPT = "application/xml"


# Logger Class Instance
logger = Swrba_Logger.new("swrba_vserver_putsnapshot", Swrba_Def::SWRBA_FILE_LOG)
logger.warn(":[begin]")

# Input Information File Analysis Class
input = Swrba_Input.new

# Initialize
option = []

# Get Option Information
while data = ARGV.shift
  option.push(data)
end

# Check Parameter Number
if option.size != 2
  logger.warn("parameter check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check Option Form
if option[0] != "-input"
  logger.warn("option check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check File Exist
if File.exist?(option[1]) == false
  logger.warn("file exist error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Input File Name
logger.warn("Input File Name = " + option[1])

# Get Input Information
begin
  servername  = input.input_analysis(option[1], "servername")
rescue
  logger.warn("input analysis error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_RETRY
end

# Check Required
if servername == nil || servername == ""
  logger.warn("servername None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

# Swrba_Vserver class Instance
vserver = Swrba_Vserver.new(Swrba_Vserver::SWRBA_KIND_RESTORE)
# Set Servername in vserver
begin
	vserver.setVserverName(servername)
rescue => exc
	logger.warn(exc)
	logger.warn(":[setVserverName error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end

begin
	managehost = nil
	managehost = vserver.getManageHost
rescue => exc
	logger.warn(exc)
	logger.warn(":[getManageHost error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end
begin
	manageport = nil
	manageport = vserver.getManagePort.to_s
rescue => exc
	logger.warn(exc)
	logger.warn(":[getManagePort error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end
logger.warn("Managername = " + managehost)
logger.warn("Managerport = " + manageport)

# Get Power Status
begin
  powerstatus = vserver.getPowerStatus
rescue => exc
	logger.warn(exc)
	logger.warn(":[getPowerStatus error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end
if powerstatus == "on"
	logger.warn(":[PowerStatus error (PowerStatus = on)]")
	errmsg = vserver.create_errmsg(Swrba_Vserver::SWRBA_RETURNCODE192, nil)
	STDERR.puts errmsg.toutf8
	exit Swrba_Vserver::SWRBA_RETURNCODE192
end

# Get Snapshot-ID
begin
	snapshotID = nil
	snapshotID = vserver.getSnapshotID
	logger.warn("snapshotID = " + snapshotID)
rescue => exc
	logger.warn(exc)
	logger.warn(":[getSnapshotID error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end
resourcepath = nil
resourcepath = RESOURCEPATH_HEAD + snapshotID + RESOURCEPATH_TAIL

# REST Communication
begin
	result_xml = nil
	result_xml = vserver.requestREST(METHODNAME, resourcepath,
	PARMS, PROXY, PROXYPORT, PROTOCOLTYPE, REQUESTXML, CONTENTTYPE, ACCEPT)
rescue => exc
	# network exception
	logger.warn(exc)
	logger.warn(":[requestREST error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end

## get Result
begin
	result_xml = vserver.rest_waitsync()
rescue => exc
	# putsnapshot vserver exception
	logger.warn(exc)
	logger.warn(":[rest_waitsync error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end


# successful
STDOUT.print servername.toutf8
STDERR.puts vserver.err_msg.toutf8
logger.warn(":[end]")

exit Swrba_Def::NORMAL_CODE
