#!/usr/bin/ruby

require 'kconv'
require 'standard/swrba_logger'           # Logger Class
require 'standard/swrba_def'              # Common Definition Class
require 'standard/swrba_input'            # Input Information File Analysis Class
require 'swrba_vserver'                   # Vserver Class
require 'swrba_vserverlist'               # Vserver List Class
require 'swrba_vserver_msgdef'            # vserver message 

#=Combination ROR
#Version::   1.0
#Copyright:: Copyright (C) FUJITSU LIMITED 2010.
#
#=== Function Contents
#* Get Virtual Server detail
#


# Const value

METHODNAME = "GET"
RESOURCEPATH_HEAD = "/l_servers/" 
PARMS = nil
PROXY = nil
PROXYPORT = 0
PROTOCOLTYPE = "https"
REQUESTXML = nil
CONTENTTYPE = "application/xml"
ACCEPT = "application/xml"


# Logger Class Instance
logger = Swrba_Logger.new("swrba_vserver_getdetail", Swrba_Def::SWRBA_FILE_LOG)
logger.warn(":[begin]")

# Input Information File Analysis Class
input = Swrba_Input.new

# Initialize
option = []

# Get Option Information
while data = ARGV.shift
  option.push(data)
end

# Check Parameter Number
if option.size != 2
  logger.warn("parameter check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check Option Form
if option[0] != "-input"
  logger.warn("option check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check File Exist
if File.exist?(option[1]) == false
  logger.warn("file exist error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Input File Name
logger.warn("Input File Name = " + option[1])

# Get Input Information
begin
  servername  = input.input_analysis(option[1], "servername")
  detail  = input.input_analysis(option[1], "detail")
rescue
  logger.warn("input analysis error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_RETRY
end

# Check Required
if servername == nil || servername == ""
  logger.warn("servername None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end
if detail == nil || detail == ""
  logger.warn("detail None")
  logger.warn("set detail (default:OFF)")
  detail = "OFF"
end

# Set detail-config
detail = detail.upcase
if detail != "ON" && detail != "OFF"
	logger.warn("detail error")
	logger.warn(":[error]")
	STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
	exit Swrba_Def::ERROR_INPUT_DATA
end

# Swrba_Vserver class Instance
vserver = Swrba_Vserver.new(Swrba_Vserver::SWRBA_KIND_DETAIL)
# Set Servername in vserver
begin
	vserver.setVserverName(servername)
rescue => exc
	logger.warn(exc)
	logger.warn(":[setVserverName error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end

begin
	managehost = nil
	managehost = vserver.getManageHost
rescue => exc
	logger.warn(exc)
	logger.warn(":[getManageHost error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end
begin
	manageport = nil
	manageport = vserver.getManagePort.to_s
rescue => exc
	logger.warn(exc)
	logger.warn(":[getManagePort error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end
logger.warn("Manager name = " + managehost)
logger.warn("Manager port = " + manageport)

## Get Resource-ID
begin
	resourceID = vserver.getResourceID
	logger.warn("resourceID = " + resourceID)
rescue => exc
	logger.warn(exc)
	logger.warn(":[getResourceID error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end
resou = nil
resou = RESOURCEPATH_HEAD + resourceID

# REST Communication(get Result)
begin
	result_xml = nil
	result_xml = vserver.requestREST(METHODNAME, resou, 
	PARMS, PROXY, PROXYPORT, PROTOCOLTYPE, REQUESTXML, CONTENTTYPE, ACCEPT)
rescue => exc
	logger.warn(exc)
	logger.warn(":[requestREST error]")
	STDERR.puts vserver.err_msg.toutf8
	exit vserver.return_code
end

# Successful
begin
	vserverlist = nil
	vserverlist = Swrba_Vserverlist.new(Swrba_Vserver::SWRBA_KIND_DETAIL)
	result_list = vserverlist.convertRORdetail(result_xml, detail)
rescue => exc
	logger.warn(exc)
	logger.warn(":[convertRORdetail error]")
	STDERR.puts vserverlist.err_msg.toutf8
	exit vserverlist.return_code
end

errmsg = nil
errmsg = vserver.create_errmsg(Swrba_Def::NORMAL_CODE, nil)
STDOUT.print result_list.toutf8
STDERR.puts errmsg.toutf8
logger.warn(":[end]")

exit Swrba_Def::NORMAL_CODE

