#!/usr/bin/ruby

require 'kconv'
require 'standard/swrba_logger'           # Logger Class
require 'standard/swrba_def'              # Common Definition Class
require 'standard/swrba_input'            # Input Information File Analysis Class
require 'swrba_vserver'                   # Vserver Class
require 'swrba_vserver_msgdef'            # vserver message 

#= vitual server delete
#Version::   1.0
#Copyright:: Copyright (C) FUJITSU LIMITED 2010.
#
#=== Function Contents
#* start virtual server
#

#
# Const value
#
METHODNAME = "DELETE"
RESOURCEPATH = "/l_servers/" 
PARMS = nil
PROXY = nil
PROXYPORT = 0
PROTOCOLTYPE = "https"
REQUESTXML = nil
CONTENTTYPE = "application/xml"
ACCEPT = "application/xml"

# Logger Class Instance
logger = Swrba_Logger.new("swrba_vserver_delete", Swrba_Def::SWRBA_FILE_LOG)
logger.warn(":[begin]")

# Input Information File Analysis Class
input = Swrba_Input.new

# Initialize
option = []

# Get Option Information
while data = ARGV.shift
  option.push(data)
end

# Check Parameter Number
if option.size != 2
  logger.warn("parameter check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check Option Form
if option[0] != "-input"
  logger.warn("option check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check File Exist
if File.exist?(option[1]) == false
  logger.warn("file exist error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Input File Name
logger.warn("Input File Name = " + option[1])

# Get Input Information
begin
  servername = input.input_analysis(option[1], "servername")
rescue
  logger.warn("input analysis error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_RETRY
end

# Check Required
if servername == nil || servername == ""
  logger.warn("servername None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

# create swrba_vserver object
vserver = Swrba_Vserver.new(Swrba_Vserver::SWRBA_KIND_DELETE)

begin
  # Get Information(Manager-Server) from CMDB
  vserver.setVserverName(servername)
  logger.warn("Managername = " + vserver.getManageHost)
  logger.warn("Managerport = " + vserver.getManagePort.to_s)
rescue => exc
  logger.warn(exc)
  logger.warn(":[setVserverName error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end

## Get Resource-ID
resourceID = nil
begin
  resourceID = vserver.getResourceID()
  logger.warn("resourceID = " + resourceID)
rescue => exc
  #  exception
  logger.warn(exc)
  logger.warn(":[getResourceID error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end

################
## Get Power Status
################
powerstatus = nil
begin
  powerstatus = vserver.getPowerStatus()
rescue => exc
  #  exception
  logger.warn(exc)
  logger.warn(":[getPowerStatus error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end

msg = nil
if powerstatus == "on"
  logger.warn("virtual server status is running")
  msg = vserver.create_errmsg(Swrba_Vserver::SWRBA_RETURNCODE192, nil)
  STDERR.puts msg.toutf8
  exit Swrba_Vserver::SWRBA_RETURNCODE192
end

################
## Delete  VServer
################
return_xml = nil
begin
  return_xml = vserver.requestREST(
                 METHODNAME, RESOURCEPATH + resourceID, PARMS, PROXY, 
                 PROXYPORT, PROTOCOLTYPE, REQUESTXML, CONTENTTYPE, ACCEPT)
rescue => exc
  #  exception
  logger.warn(exc)
  logger.warn(":[requestREST error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end

################
## Wait Delete VServer
################
return_xml = nil
begin
  return_xml = vserver.rest_waitsync()
rescue => exc
  # exception
  logger.warn(exc)
  logger.warn(":[rest_waitsync error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end

# successful
STDOUT.print servername.toutf8
STDERR.puts vserver.err_msg.toutf8
logger.warn(":[end]")

exit Swrba_Def::NORMAL_CODE
