#!/usr/bin/ruby

require 'kconv'
require 'socket'
require 'csv'
require 'standard/swrba_logger'        # Logger Class
require 'standard/swrba_def'           # Common Definition Class
require 'standard/swrba_input'         # Input Information File Analysis Class
require 'standard/swrba_util'          # Util Library class
require 'swrba_vserver'                # Vserver Class
require 'swrba_vserver_msgdef'         # vserver message 


#=vitual server create
#Version::   1.0
#Copyright:: Copyright (C) FUJITSU LIMITED 2010.
#
#=== Function Contents
#* start virtual server
#

#
# Const value
#
METHODNAME = "POST"
RESOURCEPATH = "/l_servers" 
PARMS = nil
PROXY = nil
PROXYPORT = 0
PROTOCOLTYPE = "https"

CONTENTTYPE = "application/xml"
ACCEPT = "application/xml"


SWRBA_CREATE_IP_HEAD="<Resources><LServer name='%s'>" +
"<TemplateLink id='%s' /><ServerImageLink id='%s' />"  +
"<CPU><CPUPerf>%s</CPUPerf><NumOfCPU>%s</NumOfCPU></CPU>"  +
"<Memory><MemorySize>%s</MemorySize></Memory><NICs>"


SWRBA_CREATE_IPAUTO_XML= "<NIC><NetworkLink id='%s' />" + 
"<NICIndex>%d</NICIndex><IpAddress auto='true'></IpAddress></NIC>"  

SWRBA_CREATE_IP_XML="<NIC><NetworkLink id='%s' /><NICIndex>%d</NICIndex>" +
"<IpAddress>%s</IpAddress></NIC>"

SWRBA_CREATE_IP_TAIL="</NICs><OSSetting><AdminPassword>%s</AdminPassword>" + 
"</OSSetting></LServer></Resources>"

# Logger Class Instance
logger = Swrba_Logger.new("swrba_vserver_create", Swrba_Def::SWRBA_FILE_LOG)

logger.warn(":[begin]")

# Input Information File Analysis Class
input = Swrba_Input.new

# Initialize
option = []

# Get Option Information
while data = ARGV.shift
  option.push(data)
end

# Check Parameter Number
if option.size != 2
  logger.warn("parameter check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check Option Form
if option[0] != "-input"
  logger.warn("option check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check File Exist
if File.exist?(option[1]) == false
  logger.warn("file exist error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Input File Name
logger.warn("Input File Name = " + option[1])

# Get Input Information
begin
  templatename  = input.input_analysis(option[1], "templatename")
  servername = input.input_analysis(option[1], "servername")
  serverpassword = input.input_analysis(option[1], "serverpassword")
  network = input.input_analysis(option[1], "network")
  manageserver = input.input_analysis(option[1], "manageserver")
  numofprocessor = input.input_analysis(option[1], "numofprocessor")
  processorspeed = input.input_analysis(option[1], "processorspeed")
  memorysize = input.input_analysis(option[1], "memorysize")
  imagename  = input.input_analysis(option[1], "imagename")
  serveripaddress  = input.input_analysis(option[1], "serveripaddress")
rescue
  logger.warn("input analysis error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_RETRY
end

# Check Required
if templatename == nil || templatename == ""
  logger.warn("template None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

if imagename == nil || imagename == ""
  logger.warn("imagename None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

if servername == nil || servername == ""
  logger.warn("servername None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

if serverpassword == nil || serverpassword == ""
  logger.warn("serverpassword None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

if network == nil || network == ""
  logger.warn("network None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end
networks = CSV.parse(network)
if networks[0].size > 3
  logger.warn("network too requests")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end
i = 0
while i < networks[0].size
  if networks[0][i] == nil || networks[0][i] == ""
    logger.warn("network None")
    logger.warn(":[error]")
    STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
    exit Swrba_Def::ERROR_INPUT_DATA
  end
  i = i + 1
end

if manageserver == nil || manageserver == ""
  logger.warn("manageserver None")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

serveripaddresses = nil
if serveripaddress != nil && serveripaddress != ""
  serveripaddresses = CSV.parse(serveripaddress)
  if serveripaddresses[0].size > networks[0].size
    logger.warn("serveripaddresses too requests")
    logger.warn(":[error]")
    STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
    exit Swrba_Def::ERROR_INPUT_DATA
  end
  i = 0
  while i < serveripaddresses[0].size
    if serveripaddresses[0][i] == nil || serveripaddresses[0][i] == ""
      i = i + 1
      next
    end
    isipaddress = Swrba_Util.check_ipaddress(serveripaddresses[0][i])
    if isipaddress  != 1
      logger.warn("serveripaddress error")
      logger.warn(":[error]")
      STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
      exit Swrba_Def::ERROR_INPUT_DATA
    end
    i = i + 1
  end
else
  serveripaddresses = nil
end

begin
  if numofprocessor != nil && numofprocessor != ""
    numofprocessor_arg = numofprocessor.to_s.split(/\./)
    if numofprocessor_arg.size != 1
      raise
    end
    if numofprocessor.to_i < 1 || numofprocessor.to_i > 8
      raise
    end
    changeflg = 1
  else
    numofprocessor = ""
  end
rescue
  logger.warn("numofprocessor invalid")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end


begin
  if processorspeed != nil && processorspeed != ""
    processorspeed_arg = processorspeed.to_s.split(/\./)
    if processorspeed_arg.size == 1
      if processorspeed_arg[0].to_i < 1 || processorspeed_arg[0].to_i > 8
         raise
      end
    elsif processorspeed_arg.size == 2
      if processorspeed_arg[0].to_i < 0 || processorspeed_arg[0].to_i > 8
         raise
      end
      if processorspeed_arg[1].to_i < 0 || processorspeed_arg[1].to_i > 9
         raise
      end
      if processorspeed_arg[0].to_i == 0 && processorspeed_arg[1].to_i == 0
         raise
      end
      if processorspeed_arg[0].to_i == 8 && processorspeed_arg[1].to_i > 0
         raise
      end
      if processorspeed_arg[1].to_s.length > 1
         raise
      end
    else
      raise
    end
    changeflg = 1
  else
    processorspeed = ""
  end
rescue
  logger.warn("processorspeed invalid")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

begin
  if memorysize != nil && memorysize != ""
    memorysize_arg = memorysize.to_s.split(/\./)
    if memorysize_arg.size == 1
      if memorysize_arg[0].to_i < 1 || memorysize_arg[0].to_i > 255
         raise
      end
    elsif memorysize_arg.size == 2
      if memorysize_arg[0].to_i < 0 || memorysize_arg[0].to_i > 255
         raise
      end
      if memorysize_arg[1].to_i < 0 || memorysize_arg[1].to_i > 9
         raise
      end
      if memorysize_arg[0].to_i == 0 && memorysize_arg[1].to_i == 0
         raise
      end
      if memorysize_arg[0].to_i == 255 && memorysize_arg[1].to_i > 0
         raise
      end
      if memorysize_arg[1].to_s.length > 1
         raise
      end
    else
      raise
    end
    changeflg = 1
  else
    memorysize = ""
  end
rescue
  logger.warn("memorysize invalid")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

#####################
# create swrba_vserver class
#####################
manageipaddress = nil
begin
  manageipaddress = IPSocket.getaddress(manageserver)
rescue
  logger.warn("servername convert error")
  logger.warn(":[error]")
  STDERR.puts Swrba_Vserver_Msgdef::SWRBA_MSG_ERR_ARGUMENT
  exit Swrba_Def::ERROR_INPUT_DATA
end

  vserver = Swrba_Vserver.new(Swrba_Vserver::SWRBA_KIND_CREATE)
begin
  vserver.setManageHost(manageipaddress)
  vserver.setVserverName(servername)
  logger.warn("Managername = " + vserver.getManageHost)
  logger.warn("Managerport = " + vserver.getManagePort.to_s)
rescue => exc
  logger.warn(exc)
  logger.warn(":[setManageHost error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end

################
## Get Template ID
################
templateID = nil
begin
  templateID = vserver.getTemplateID(templatename)
rescue => exc
  #  exception
  logger.warn(exc)
  logger.warn(":[getTemplateID error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end


################
## Get Image ID
################
imageID = nil
begin
  imageID = vserver.getImageID(imagename)
rescue => exc
  # exception
  logger.warn(exc)
  logger.warn(":[getImageID error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end

xml_str_tmp = nil
xml_str_tmp = sprintf(SWRBA_CREATE_IP_HEAD,servername ,templateID, imageID,
                 processorspeed.to_s, numofprocessor.to_s, memorysize.to_s)
xml_str = xml_str_tmp
i = 0
while i < networks[0].size
  ##################
  ## Get Network ID
  ##################
  netID = nil
  begin
    netID = vserver.getNetworkID(networks[0][i])
  rescue => exc
    # exception
    logger.warn(exc)
    logger.warn(":[getNetworkID error]")
    STDERR.puts vserver.err_msg.toutf8
    exit vserver.return_code
  end
  if serveripaddresses == nil || serveripaddresses[0].size <= i ||
     serveripaddresses[0][i] == nil || serveripaddresses[0][i] == ""
    ## Create XML file
    xml_str_tmp = sprintf(SWRBA_CREATE_IPAUTO_XML, netID, i)
  else
    ## Create XML file
    xml_str_tmp = sprintf(SWRBA_CREATE_IP_XML, netID,i,serveripaddresses[0][i])
  end
  
  xml_str = xml_str + xml_str_tmp
  
  i = i + 1
end

xml_str_tmp = sprintf(SWRBA_CREATE_IP_TAIL, serverpassword.to_s)
xml_str = xml_str + xml_str_tmp

################
## Create VServer
################
return_xml = nil
begin
  return_xml = vserver.requestREST(
                 METHODNAME, RESOURCEPATH, PARMS, PROXY, 
                 PROXYPORT, PROTOCOLTYPE, xml_str, CONTENTTYPE, ACCEPT)
rescue => exc
  # exception
  logger.warn(exc)
  logger.warn(":[requestREST error]: ")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end


#####################
## Wait Create VServer
#####################
return_xml = nil
begin
  return_xml = vserver.rest_waitsync()
rescue => exc
  # exception
  logger.warn(exc)
  logger.warn(":[rest_waitsync error]")
  STDERR.puts vserver.err_msg.toutf8
  exit vserver.return_code
end
    
# successful
STDOUT.print servername.toutf8
STDERR.puts vserver.err_msg.toutf8
logger.warn(":[end]")

exit Swrba_Def::NORMAL_CODE
